/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ws3dproxy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;
import ws3dproxy.model.Creature;
import ws3dproxy.model.Leaflet;
import ws3dproxy.model.Thing;
import ws3dproxy.model.WorldPoint;
import ws3dproxy.util.Constants;

/**
 * Utility class to send commands to the server. The client sends commands to
 * the server in order to control the creature. Each command corresponds to an
 * action that the creature must perform or a request to change a specific
 * attribute of the creature.
 *
 * @author ecalhau
 */
public class CommandUtility {

    public static final String ERROR_CODE = "@@@";
    //public Status newStatus = Status.getInstance();
    private static final Logger logger = Logger.getLogger("CommandUtility");
    private static double selfX = 0.0, selfY = 0.0, wheel = 0.0, speed = 0.0, mySelfPitch = 0.0, fuel = 0.0, size = 0.0;
    private static String myselfColor = "No color yet", index = "none";
    private static String myselfName = "none";
    private static int motorSys = 2;
    private static int hasLeaflet = 0;
    private static Creature creature;

    /**
     * Command to create a visual reference (e.g. arrow) at a specific point
     * within the server environment and expressed in Cartesian coordinate
     * system (x,y). Each point within the server environment is called a
     * "waypoint"
     *
     * @param x abscissa of the point
     * @param y ordinate of the point
     * @return server response: Point (x, y) where the waypoint was created.
     * Format: "x y"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewWaypoint(double x, double y) throws CommandExecException {

        SocketUtility.show("Sending waypoint: x= " + x + " y= : " + y);

        String controlMessage = "newwp " + x + " " + y;//newwp <X> <Y>
        return sendCmdAndGetResponse(controlMessage);

    }

    /**
     * Command to set the simulation game environment dimension: width, height
     * and the texture for the simulation floor.
     *
     * @param width width of the simulation environment
     * @param height height of the simulation environment
     * @return report from server in format: "width height pathToFloorTexture"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendEnvironmentDimension(int width, int height) throws CommandExecException {

        SocketUtility.show("Sending dimension: width= " + width + " height= : " + height);

        String controlMessage = "setenv " + width + " " + height;//setenv <width> <height>
        return sendCmdAndGetResponse(controlMessage);

    }

    /**
     * Command to delete the visual reference (arrow) which indicates a location
     * (waypoint) within the server environment.
     *
     * @param x x abscissa of the waypoint
     * @param y y ordinate of the waypoint
     * @return report from server: position of the deleted waypoint. Format "x
     * y"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendDelWaypoint(double x, double y) throws CommandExecException {
        SocketUtility.show("Delete waypoint: x= " + x + " y= : " + y);

        String controlMessage = "delwp " + x + " " + y;//delwp <X> <Y>
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Command to start the motor system of the creature.
     *
     * @return report from server. Format: "Run creature run..."
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendStartCreature(String robotID) throws CommandExecException {
        String controlMessage = "start " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Command to stop the motor system of the creature.
     *
     * @return report from server. Format: "Creature has stopped!!!"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendStopCreature(String robotID) throws CommandExecException {
        String controlMessage = "stop " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Send the creature to a specific waypoint (location).
     *
     * The angle of the turn is evaluated according to the direction established
     * between the points: robot location and (X,Y) position (waypoint). The
     * creature speed (i.e. linear velocity) is the arithmetic mean of Vr and
     * Vl.
     *
     * @param vr linear velocity of the right wheel
     * @param vl linear velocity of the left wheel
     * @param x abscissa of the waypoint
     * @param y ordinate of the waypoint
     * @return report from server. Format: "speed pitch"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGoTo(String robotID, double vr, double vl, double x, double y) throws CommandExecException {
        SocketUtility.show("Sending speeds: Vr= " + vr + " Vl= " + vl);
        String controlMessage = "setGoTo " + robotID + " " + vr + " " + vl + " " + x + " " + y;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Set the rotation around the creature's axis of symmetry. The creature's
     * movement is changed in order to make it faces towards a certain
     * direction. The creature speed (i.e. linear velocity) is the arithmetic
     * mean of Vr and Vl. The angle must be in rad.
     *
     * @param vr linear velocity of the right wheel
     * @param vl linear velocity of the left wheel
     * @param w the new pitch of the creature in rad
     * @return report from server. Format: "speed pitch"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendSetAngle(String robotID, double vr, double vl, double w) throws CommandExecException {
        SocketUtility.show("Sending speeds: Vr= " + vr + " Vl= " + vl + " and w= " + w);

        String controlMessage = "setAngle " + robotID + " " + vr + " " + vl + " " + w;//set <ID> <Vv> <Vl> <w>
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * This command makes the creature turn a specific angle. The angle of the
     * turn is derived from the linear velocity of the wheels (Vr-right
     * Vl-left). The velocity of the creature's overall movement corresponds to
     * the speed parameter.
     *
     * Note: The final pitch of the robot depends on its original orientation
     * (pitch). HINTs: +~30deg: Vl= 11 and Vr= 0.6; +~45deg: Vl= 16 and Vr= 0.2;
     * +~60deg: Vl= 22 and Vr= 1;+~90deg: Vl= 32 and Vr= 0.6; +~180deg: Vl= 63
     * and Vr= 0.2; +~270deg: Vl= 95 and Vr= 0.8.
     *
     *
     * @param robotID sequence number assigned to each object once it is
     * created.
     * @param speed velocity of the creature's overall motion
     * @param vr right wheel velocity
     * @param vl left wheel velocity
     * @return report from server. Format: "speed pitch[rad] pitch[deg]"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendSetTurn(String robotID, double speed, double vr, double vl) throws CommandExecException {
        SocketUtility.show("Sending speeds: Vr= " + vr + " Vl= " + vl + " and speed= " + speed);

        String controlMessage = "setTurn " + robotID + " " + speed + " " + vr + " " + vl;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Randomly generates a new leaflet. A leaflet is a list of crystals to be
     * collected and delivered at the DeliverySpot.
     *
     * @return the leaflets (3 in total). Format: "Leaflet1 Leaflet2 Leaflet3"
     * Leaflet format: "[Color TotalNumber NumberCollected] Payment Example:
     * White 2 0 Yellow 1 0 6 White 1 0 Magenta 2 0 5 Blue 1 0 Red 1 0 Green 1 0
     * 24 Leaflet1: 2 white jewels and 1 yellow jewel. None already collected.
     * This leaflet increases the score in 6 points Leaflet2: 1 white jewel and
     * 2 magenta jewels. None already collected. This leaflet increases the
     * score in 5 points Leaflet3: 1 Blue, 1 Red and 1 green jewel. None already
     * collected. This leaflet increases the score in 24 points
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGenLeaflet() throws CommandExecException {
        //randomly generates a new leaflet:
        String controlMessage = new String("leaflet");
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * The leaflet is delivered at the DeliverySpot. All the crystals specified
     * in the leaflet were collected and the creature is at the DeliverySpot.
     *
     * @return report from the server. Format: "Leaflet delivered!"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendDeliverLeaflet(String robotID, String leafletID) throws CommandExecException {
        //randomly generates a new leaflet:
        String controlMessage = "deliver " + robotID + " " + leafletID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * The creature grasp and put an object in its sack. The object may be: food
     * or crystal.
     *
     * @param thingName the unique name of the Thing. Example:
     * Jewel_1363904023537
     * @return report from server with the name of the grasped Thing. Format:
     * "ThingName".
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendPutInSack(String robotID, String thingName) throws CommandExecException {
        SocketUtility.show("Sending thing name: " + thingName);

        String controlMessage = new String("sackit " + robotID + " " + thingName);//old graspit
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * The creature eats a food.
     *
     * @param thingName the unique name of the Thing. Example:
     * Jewel_1363904023537
     * @return report from server with the name of the Thing which has been
     * eaten. Format: "ThingName".
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendEatIt(String robotID, String thingName) throws CommandExecException {
        SocketUtility.show("Sending food name: " + thingName);

        String controlMessage = "eatit " + robotID + " " + thingName;//eatit <ID> <thing name>
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Command to drop a Thing which is in creature's sack.
     *
     * @param robotID sequence number assigned to each object once it is
     * @param type 3-Jewel; 21-Perishable food; 22- Non-Perishable food
     * @param color 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @return report from server with the point (x,y) where the Thing has been
     * dropped. Format: "x y".
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendDropIt(String robotID, int type, int color) throws CommandExecException {
        SocketUtility.show("Sending drop: type= " + type + " color= " + color);
        //drop <CreaturePoolIndex> <Type> <Thing color (for jewels) or any number for food> - Type: 3-Jewel;
        String controlMessage = new String("drop " + robotID + " " + type + " " + color);
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * The creature grasp and hide an object under the ground.
     *
     * @param thingName the unique name of the Thing. Example:
     * Jewel_1363904023537
     * @return report from server with the name of the Thing which has been
     * hidden. Format: "ThingName".
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendHideIt(String robotID, String thingName) throws CommandExecException {
        SocketUtility.show("Sending thing name: " + thingName);

        String controlMessage = "hideit " + robotID + " " + thingName;//hideit <ID> <thing name>
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * The creature digs an object up. The object must have been hided before.
     *
     * @see #sendHideIt
     *
     * @param thingName the unique name of the Thing. Example:
     * Jewel_1363904023537
     * @return report from server with the name of the Thing which has been
     * "unearthed". Format: "ThingName".
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendUnhideIt(String robotID, String thingName) throws CommandExecException {
        SocketUtility.show("Sending thing name: " + thingName);

        String controlMessage = new String("unhideit " + robotID + " " + thingName);//unhideit <ID> <thing name>
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Check if the simulation on the server has started.
     *
     * @return "true": has started; "false": not started yet.
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static boolean requestGameStatus() throws CommandExecException {
        boolean ret = false;
        SocketUtility.show("Sending request if game has started.");

        String controlMessage = "game";
        SocketUtility.sendMessage(controlMessage);
        String returnMessage = SocketUtility.receiveMessage();
        checkIfErrorMessage(returnMessage);
        SocketUtility.show("----->>>>Server response: " + returnMessage);

        if (returnMessage.equals("yes")) {
            ret = true;
        }
        SocketUtility.show("----->>>> " + ret);
        return ret;
    }

    /**
     * Check if the specified creature has been already created on the server.
     *
     * @param robotID an integer that is a sequence number assigned to each
     * object once it is created. It specifies a creature.
     * @return true if creature exists; false otherwise. 
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static boolean ifCreatureExists(String robotID) throws CommandExecException {
        boolean ret = false;
        String firstParam = "";
        //command "check" returns two parameters. First: true if creature exists; false otherwise. 
        // Second: creatureNameID
        String controlMessage = "check " + robotID;
        StringTokenizer st = sendCmdAndGetResponse(controlMessage);
        if (st.hasMoreTokens()) {
            firstParam = st.nextToken();
            if (firstParam.equals("yes")) {
                ret = true;
            }
        }if (st.hasMoreTokens()) {
            myselfName = st.nextToken();
        }

        return ret;
    }

    /**
     * Check if the specified creature has been already created on the server at
     * a certain position. If it already exists but with different pitch, the pitch
     * is changed.
     *
     * @param X x of position of creature
     * @param Y y of position of creature
     * @param pitch pitch of position of creature
     * @return "": does not exist; otherwise: creatureIndex creatureNameID
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static String checkCreature(double X, double Y, double pitch) throws CommandExecException {
        String controlMessage = "checkXY " + X + " " + Y + " " + pitch;
        SocketUtility.sendMessage(controlMessage);
        String msg = SocketUtility.receiveMessage();
        if (!msg.equals("")) {
            checkIfErrorMessage(msg);
        }
        return msg;
    }

    /**
     * Delete all entities in the world (creatures and objects).
     * @return "done" if successful
     * @throws CommandExecException 
     */
    public static String sendResetWorld() throws CommandExecException {
        String controlMessage = "worldReset ";
        SocketUtility.sendMessage(controlMessage);
        String msg = SocketUtility.receiveMessage();
        if (!msg.equals("")) {
            checkIfErrorMessage(msg);
        }
        return "done";
    }

    /**
     * A name that identifies the client. Examples: RobotMind_0, RobotMind_1
     * etc.
     *
     * @param robotID sequence number assigned to each object once it is
     * created.
     * @return report from server with the "mind name". Format: "MindName"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer setMindName(String robotID) throws CommandExecException {
        String controlMessage = "mindName " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Starts the camera (visual sensor) of the creature.
     *
     * Note: Currently, there are only 2 cameras available on the server. The
     * right camera is attached to even robotIDs and the left camera, to the odd
     * robotIDs.
     *
     * @param robotID sequence number assigned to each object once it is
     * created.
     * @return report from server with the "creature index" indicating the
     * current owner of a camera. Even indexes are attached to right camera and
     * odd indexes to the left one. Example: camera 0 0
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendStartCamera(String robotID) throws CommandExecException {
        String controlMessage = "camera " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Get the values of the simulation parameters: width, height, xDS, yDS
     * width and height of the environment of the game (simulation) (xDS, yDS)
     * location of the DeliverySpot.
     *
     * @return a string in the format "width height xDS yDS"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetSimulationParameters() throws CommandExecException {
        String controlMessage = "getsimulpars";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the amount of free memory in the Java Virtual Machine.
     *
     * @return a string indicating an approximation to the total amount of
     * memory currently available for future allocated objects, measured in
     * bytes.
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetFreeMemory() throws CommandExecException {
        String controlMessage = "memory";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the current time in milliseconds.
     *
     * @return a string indicating the difference, measured in milliseconds,
     * between the current time and midnight, January 1, 1970 UTC.
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetClock() throws CommandExecException {
        String controlMessage = "getclock";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Command to get the creature position.
     *
     * @param robotID sequence number assigned to each object once it is
     * created.
     * @return the creature position in the format: "X Y Pitch" Example: 287.0
     * 391.0 0.0
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetCreaturePosition(String robotID) throws CommandExecException {
        String controlMessage = "getcreatcoords " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the creature's speed and fuel current values.
     *
     * @param robotID sequence number assigned to each object once it is
     * created.
     * @return creature's speed and fuel values in the format: "speed fuel"
     * Example: 1.0 1000
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetCreatureInfo(String robotID) throws CommandExecException {
        String controlMessage = "getcreatinfo " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the affordances of a Thing. The server respond with the list of
     * affordances code. Codes: Affordance__VIEWABLE = 30; Affordance__HIDEABLE
     * = 31; Affordance__UNHIDEABLE = 32; Affordance__GRASPABLE = 33;
     * Affordance__EATABLE = 34; Affordance__PUTINBAGABLE = 35;//sth that can be
     * put in a bag; Affordance__OPENABLE = 36; //sth that can be opened (eg. a
     * cage); Affordance__CLOSEABLE = 37;//sth than be closed (eg. a cage);
     * Affordance__INSERTABLE = 38;//sth than can contain another thing (which
     * had been inserted into this sth)[eg. a container];
     * Affordance__REMOVEFROMABLE = 39;//sth from whose inside another thing is
     * removed (eg. a container)
     *
     * @param thingID refers to a ThingID. Example: Brick_1364939987440
     * @return list of affordance codes. Example: 30 31 32 Meaning:
     * Affordance__VIEWABLE Affordance__HIDEABLE Affordance__UNHIDEABLE
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetAffordances(String thingID) throws CommandExecException {
        String controlMessage = "getcreatcoords " + thingID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the simulation environment dimension.
     *
     * @return environment dimension in the format: "width height". Example: 800
     * 600
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetEnvironmentDimension() throws CommandExecException {
        String controlMessage = "getenvironmen ";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the number of creatures and other Things currently at the
     * simulation environment.
     *
     * @return server response with the number of entities. Format:
     * "NumCreatures NumOtherThings"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetNumberOfEntities() throws CommandExecException {
        String controlMessage = "getNumEntities ";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the creatures and other Things currently at the simulation
     * environment.
     *
     * @return server response with the entities. Format: "numberOfEntities
     * ThingName1 category isOccluded X1 Y1 X2 Y2 pitch colorIndex
     * ThingName2..." Example: 3 Creature_1364966079239 0 0 275.0 275.0 373.0
     * 373.0 0 Jewel_1364966084869 3 0 552.0 552.0 355.0 355.0 1
     * NPFood_1364966088030 22 0 469.0 469.0 311.0 311.0 1
     *
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetWorldEntities() throws CommandExecException {
        String controlMessage = "getall ";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Returns the Thing which is closest to a creature.
     *
     * @param robotID sequence number assigned to each object once it is
     * @return server response with the number of entities. Format: "ThingName1
     * category isOccluded X1 X2 Y1 Y2 pitch hardness energy taste colorName"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetClosest(String robotID) throws CommandExecException {
        String controlMessage = "closest";
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Create a new creature.
     *
     * @param x abscissa of the location of the creature
     * @param y ordinate of the location of the creature
     * @param pitch direction to which the creature is headed
     * @return server successful response with creature position. Format:
     * "CreatureIndex CreatureName X Y Pitch"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewCreature(double X, double Y, double pitch) throws CommandExecException {
        String controlMessage = "new " + X + " " + Y + " " + pitch;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creates a new cage.
     *
     * @param x abscissa of the location of the cage
     * @param y ordinate of the location of the cage
     * @return server successful response with cage position. Format: "CageName
     * X Y"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewCage(double X, double Y) throws CommandExecException {
        String controlMessage = "cage " + X + " " + Y;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creates a new food entity.
     *
     * @param type 0-perishable 1-non-perishable
     * @param x abscissa of the location of the food
     * @param y ordinate of the location of the food
     * @return server successful response with food position. Format: "FoodName
     * X Y"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewFood(int type, double X, double Y) throws CommandExecException {
        String controlMessage = "food " + type+" "+X + " " + Y;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creates several food at once.
     *
     * @param type 0-perishable 1-non-perishable
     * @param number number of food entities to create
     * @param X_Y text with the list of location where a new food must be
     * created
     * @return server successful response with food names and positions. Format:
     * "FoodName0 X Y FoodName1 X Y ..."
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewFoodInBatch(int type, int number, String X_Y) throws CommandExecException {
        String controlMessage = "batchoffood " + type + " " + number + " " + X_Y;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creates several jewels at once.
     *
     * @param type 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @param number number of jewel entities to create
     * @param X_Y text with the list of location where a new jewel must be
     * created
     * @return server successful response with jewel names and positions.
     * Format: "JewelName0 X Y JewelName1 X Y ..."
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewJewelsInBatch(int type, int number, String X_Y) throws CommandExecException {
        String controlMessage = "batchofjewels " + type + " " + number + " " + X_Y;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creates a new jewel entity.
     *
     * @param type 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @param x abscissa of the location of the jewel
     * @param y ordinate of the location of the jewel
     * @return server successful response with jewel position. Format:
     * "JewelName X Y"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewJewel(int type, double X, double Y) throws CommandExecException {
        String controlMessage = "jewel " + type + " "+ X + " " + Y;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creates a new brick entity.
     *
     * @param type 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @param x abscissa of the location of the brick
     * @param y ordinate of the location of the brick
     * @return server successful response with brick position. Format:
     * "BrickName X Y"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendNewBrick(int type, double X1, double Y1, double X2, double Y2) throws CommandExecException {
        String controlMessage = "brick " +type + " "+ X1 + " " + Y1 + " " + X2+ " " + Y2;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * The creature state refers to its location and camera content (list of
     * objects that have been captured).
     *
     * @param robotNameID name that uniquely identifies the creature in the system
     *
     * @return list of creature's attributes and the objects that have been
     * captured by the camera (creature's visual system).
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendGetCreatureState(String robotNameID) throws CommandExecException {
        String controlMessage = "getcreaturestate " + robotNameID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creature's energy is restored.
     *
     * @param robotID sequence number assigned to each object once it is
     * created.
     * @return creature's fuel value
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendRefuel(String robotID) throws CommandExecException {
        String controlMessage = "refuel " + robotID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Delete an entity: creature or other Thing.
     *
     * @param type 0: food, jewel or blick; 1: creature.
     * @param thingID sequence number assigned to each creature and to other
     * entities (different pools)
     * @return the name of the deleted Thing
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized StringTokenizer sendDeleteThing(int type, String thingID) throws CommandExecException {
        String controlMessage = "deleteth " + type + " " + thingID;
        return sendCmdAndGetResponse(controlMessage);
    }

    /**
     * Creature is created in mind.
     *
     *
     *
     * @return the creature created 

     */
    public static synchronized Creature initializeCreature(String indexID, String nameID) {
        myselfName = nameID;
        creature = Creature.getInstance(indexID, myselfName, myselfColor, speed, wheel, size, size, motorSys, fuel, new WorldPoint(selfX, selfY), hasLeaflet);
        return creature;
    }

    public static synchronized Creature initializeCreature(String indexID) {
        //myselfName was already initialized in ifCreatureExists
        return initializeCreature(indexID, myselfName);
    }

    /**
     * Get information regarding the creature state and the content of its
     * visual system (camera) according to a protocol established with the
     * server. The information is used to create the list of objects (Thing)
     * that are manipulated in the "mind".
     *
     * @return the status that encapsulates the objects (system entities such as
     * creature, food,jewel, brick) in the format (Thing) that they are
     * manipulated in "mind"
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */

    public static synchronized Creature updateStatus() throws CommandExecException {

        List<Leaflet> leafletList = new ArrayList<Leaflet>(); 
        HashMap<String, Integer[]> leafletItemsMap = new HashMap<String, Integer[]>();

        String command = "";

        double x1 = 0.0, x2 = 0.0, y1 = 0.0, y2 = 0.0, thingPitch = 0.0, hardness = 0.0, energy = 0.0, taste = 0.0;
        String thingName = "none";
        int category = 0;
        int isOccluded = 0;
        int number = 0; //number of Thing in visual sensor
        int numberOfLeaflets = 0;
        int numberOfLeafletItems = 0;
        long leafletID = 0;
        //int activity = 1;
        int payment = 0;
        String itemKey = "";
        Integer totalNumber = 0;
        Integer collected = 0;
        String thingColor = "No color yet";
        List<Thing>thingsInVision = new ArrayList<Thing>();

        logger.info("----------- updateThingsStatus -----------");
        //newStatus.resetThings();

        StringTokenizer st = CommandUtility.sendGetCreatureState(myselfName);

        ///////////////////Creature data:
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - myName missing!");
        } else {
            command = st.nextToken();
            myselfName = command; //string
        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - index is missing!");
        } else {
            index = st.nextToken();
        }  
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - x missing!");
        } else {
            command = st.nextToken();
            selfX = Double.parseDouble(command);

        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - y missing!");
        } else {
            command = st.nextToken();
            selfY = Double.parseDouble(command);

        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - creature size missing!");
        } else {
            command = st.nextToken();
            size = Double.parseDouble(command);

        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - pitch missing!");
        } else {
            command = st.nextToken();
            mySelfPitch = Double.parseDouble(command);
            mySelfPitch = Math.toRadians(mySelfPitch) + Constants.M_PI_2;
        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - motor system missing!");
        } else {
            command = st.nextToken();
            motorSys = Integer.parseInt(command);
        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - wheel missing!");
        } else {
            command = st.nextToken();
            wheel = Double.parseDouble(command);
        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - speed missing!");
        } else {
            command = st.nextToken();
            speed = Double.parseDouble(command);
        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - fuel missing !");
        } else {
            command = st.nextToken();
            fuel = Double.parseDouble(command);

        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - creature's color missing !");
        } else {
            command = st.nextToken();
            myselfColor = command; //string
        }
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - hasLeaflet missing !");
        } else {
            command = st.nextToken();
            hasLeaflet = Integer.parseInt(command);
        }

        if (hasLeaflet == 1) { //true
            /*
             * Read leaflets:
             */
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing number of leaflets!");
            } else {
                command = st.nextToken();
                numberOfLeaflets = Integer.parseInt(command);
            }
            for (int i = 0; i <= numberOfLeaflets - 1; i++) { //loop to create leaflets

                if (!st.hasMoreTokens()) {
                    logger.log(Level.SEVERE, "Error - missing leaflet ID!");
                } else {
                    command = st.nextToken();
                    leafletID = Long.parseLong(command);
                }

                if (!st.hasMoreTokens()) {
                    logger.log(Level.SEVERE, "Error - missing number of types in leaflet!");
                } else {
                    command = st.nextToken();
                    numberOfLeafletItems = Integer.parseInt(command);
                }
                for (int j = 0; j <= numberOfLeafletItems - 1; j++) { //loop to create a leaflet
                    if (!st.hasMoreTokens()) {
                        logger.log(Level.SEVERE, "Error - type is missing!");
                    } else {
                        command = st.nextToken();
                        itemKey = command;
                    }
                    if (!st.hasMoreTokens()) {
                        logger.log(Level.SEVERE, "Error - total number of a specific type is missing!");
                    } else {
                        command = st.nextToken();
                        totalNumber = Integer.parseInt(command);
                    }
                    if (!st.hasMoreTokens()) {
                        logger.log(Level.SEVERE, "Error - number of collected items of a type is missing!");
                    } else {
                        command = st.nextToken();
                        collected = Integer.parseInt(command);
                    }
                    Integer[] values = {0, 0};
                    values[0] = totalNumber; //total number of a type
                    values[1] = collected; //how many have been collected
                    leafletItemsMap.put(itemKey, values);
                }
                if (!st.hasMoreTokens()) {
                    logger.log(Level.SEVERE, "Error - missing payment of leaflet!");
                } else {
                    command = st.nextToken();
                    payment = Integer.parseInt(command);
                }
                leafletList.add(new Leaflet(leafletID, leafletItemsMap, payment));
                leafletItemsMap.clear();
            }

        }
        
        //update creature state in Status:
        creature.update(index, myselfColor, speed, wheel, mySelfPitch, fuel, new WorldPoint(selfX, selfY), leafletList);
        
        /**
         * Read Contact and Visual sensors!!!!!!
         */
        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - missing number of things!");
        } else {
            command = st.nextToken();
            number = Integer.parseInt(command);
        }

        for (int i = 0; i <= number - 1; i++) { //loop to create Things
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing name!");
            } else {
                command = st.nextToken();
                if (command.equals("||"))command = st.nextToken();
                thingName = command;

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing category!");
            } else {
                command = st.nextToken();
                category = Integer.parseInt(command);

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing if occluded!");
            } else {
                command = st.nextToken();
                isOccluded = Integer.parseInt(command);

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing X1!");
            } else {
                command = st.nextToken();
                x1 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing X2!");
            } else {
                command = st.nextToken();
                x2 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing Y1!");
            } else {
                command = st.nextToken();
                y1 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing Y2!");
            } else {
                command = st.nextToken();
                y2 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing pitch !");
            } else {
                command = st.nextToken();
                thingPitch = Double.parseDouble(command);
                thingPitch = Math.toRadians(thingPitch) + Constants.M_PI_2;

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing hardness!");
            } else {
                command = st.nextToken();
                hardness = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing energy!");
            } else {
                command = st.nextToken();
                energy = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing taste!");
            } else {
                command = st.nextToken();
                taste = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing color of thing!");
            } else {
                command = st.nextToken();
                thingColor = command;
            }

            thingsInVision.add(createThing(thingName, category, isOccluded, thingColor, x1, y1, x2, y2, thingPitch, hardness, energy, taste));
        }
        creature.setThingsInVision(thingsInVision);

        return creature;
    }



    private static void checkIfErrorMessage(String resp) throws CommandExecException {
        StringTokenizer st = new StringTokenizer(resp);
        if (st.hasMoreTokens()) {
            String startResp = st.nextToken();
            if (startResp.equals(ERROR_CODE)) {
                throw new CommandExecException(resp.toString());
            }
        }
    }

    private static StringTokenizer sendCmdAndGetResponse(String formattedCmd) throws CommandExecException {     
        SocketUtility.sendMessage(formattedCmd);
        String returnMessage = SocketUtility.receiveMessage();
        checkIfErrorMessage(returnMessage);
        StringTokenizer st = new StringTokenizer(returnMessage);
        SocketUtility.show("----->>>>Server response: " + returnMessage);
        return st;
    }

    private static Thing createThing(String name, int category, int ifIsOccluded, String thingColor, double x1, double y1, double x2, double y2, double thingPitch, double hardness, double energy, double taste) {

        return new Thing(name, category, x1, y1, x2, y2, thingColor, ifIsOccluded, thingPitch, hardness, energy, taste) {
        };
    }

    private static Leaflet createLeaflet(Long ID, HashMap items, int payment) {
        return new Leaflet(ID, items, payment);
    }
}
